#ifndef _ARITHMETIC_H
#define _ARITHMETIC_H

#include <stdio.h>
#include "datatype.h"
#include "des3.h"
#include "openssl/rsa.h"
#include "openssl/pem.h"
#include "openssl/sha.h"


#ifdef __cplusplus
extern "C"
{
#endif
//Enter key on the number of bits to obtain the public key file
unsigned int RSAKey_BitsOfPubPem(char* strPubPath);
unsigned int RSAKey_BitsOfPriPem(char* strPriPath);

//Determine whether the matching key pair, is true pairing success.
BOOL RSAKey_IsKeyMatch(char* strPubPath, char* strPriPath);

//Generate a RSA key pairoutput N,D,P,Q,... E==RSA_F4(0x10001L)
BOOL	RSAKey_GenerateKey(unsigned int dwBits,
						   BYTE* pN, unsigned int* pdwNLen,
						   BYTE* pD,unsigned int* pdwDLen,
						   BYTE* pP,unsigned int* pdwPLen,
						   BYTE* pQ,unsigned int* pdwQLen,
						   BYTE* pDP,unsigned int* pdwDPLen,
						   BYTE* pDQ,unsigned int* pdwDQLen,
						   BYTE* pInvQ,unsigned int* pdwInvQLen);

//Extracted from the PEM format N,D,P,Q,DP,DQ,InvQ ,if E!=RSA_F4(0x10001L)Function returns failure.
int 	RSAKey_Pem2Mid(char* strPubPath,char* strPriPath,unsigned int dwBits,
					   BYTE* pN,unsigned int* pdwNLen,
					   BYTE* pD,unsigned int* pdwDLen,
					   BYTE* pP,unsigned int* pdwPLen,
					   BYTE* pQ,unsigned int* pdwQLen,
					   BYTE* pDP,unsigned int* pdwDPLen,
					   BYTE* pDQ,unsigned int* pdwDQLen,
					   BYTE* pInvQ,unsigned int* pdwInvQLen);


BOOL	RSAKey_Mid2Pem(char* strPubPath,char* strPriPath,unsigned int dwBits,
					   BYTE* pN,unsigned int dwNLen,
					   BYTE* pD,unsigned int dwDLen,
					   BYTE* pP,unsigned int dwPLen,
					   BYTE* pQ,unsigned int dwQLen,
					   BYTE* pDP,unsigned int dwDPLen,
					   BYTE* pDQ,unsigned int dwDQLen,
					   BYTE* pInvQ,unsigned int dwInvQLen);


//! Obtained from the RSA key file path inside the
RSA* GetKeyFromFile(char* pubpath,char* pripath,unsigned int* dwBits);

//! Get the key from memory
RSA* GetKeyFromMem(BYTE*  buf);

//! RSA encrypt
BOOL RSAEncrypt(RSA* key,unsigned int dwBits,BYTE* buf,BYTE* enc);
//! RSA decrypt
BOOL RSADecrypt(RSA* key,unsigned int dwBits,BYTE* enc,BYTE* buf);

//! Find the key inside from the global list
//bool find_des_key(usb_dev_handle* g_device_handle,des_context* dt);

//! 3DES encrypt
void des3_enc(usb_dev_handle* g_device_handle,BYTE* buf);
//! 3DES decrypt
void des3_dec(usb_dev_handle* g_device_handle,BYTE* buf);
//! SHA-1 Digest Algorithm
void Sha1(BYTE* buf, unsigned int buflen, BYTE* md);
#ifdef __cplusplus
}
#endif
#endif //! _ARITHMETIC_H
