#ifndef _ROCKEY1_API_H_
#define _ROCKEY1_API_H_

#include "datatype.h"
#include "des3.h"
#include "drv.h"
#include "Arithmetic.h"
#include "libusb/usb.h"
#include "libusb/usbi.h"

/*
#pragma pack(1)
#include <setupapi.h>
#pragma pack()
*/
//semaphore and shared memory KEY value
#define   SEMAPHORE_PATH         "/root"
#define   SHARE_MEMORY_PATH      "/lib"
//#ifndef ROCKEY1_API
//#define ROCKEY1_API


#define   R1_S_SUCCESS						0x00000000	/** success*/
#define   R1_E_OPEN_FAILED					0xF0000001	/** open fail*/
#define   R1_E_NO_LIST						0xF0000002	/** not find the device list*/
#define   R1_E_KEY_REMOVED					0xF0000003	/** device disconnect or device detachment*/
#define   R1_E_INVALID_PARAMETER			0xF0000004	/** parameter fail*/
#define   R1_E_WORNG_USER_PIN				0xF0000005	/** userpasswd error*/
#define   R1_E_WORNG_SO_PIN					0xF0000006	/** vendorpasswd error*/
#define   R1_E_CHANGE_USER_PIN				0xF0000007	/** modify userpasswd fail*/
#define   R1_E_CHANGE_SO_PIN				0xF0000008	/** modify vendorpasswd fail*/
#define   R1_E_TDES_BUF_LENGTH_ERROR		0xF0000009	/** 3DES encrypt content length fail*/
#define   R1_E_RSA_BUF_LENGTH_ERROR			0xF000000A	/** RSA encrypt or decrypt content length fail*/
#define   R1_E_RSA_ID_ERROR	        		0xF000000B	/** RSA key ID error*/
#define   R1_E_RSA_ENC_ERROR        		0xF000000C	/** RSA pub key encrypt error*/
#define   R1_E_SET_UPDATE_ERROR       		0xF000000D	/** please first setup update packet*/
#define   R1_E_RSA_UPDATE_ERROR       		0xF000000E	/** make RSA update key fail*/
#define   R1_E_DES_UPDATE_ERROR       		0xF000000F	/** make 3des update key fail*/
#define   R1_E_FORMAT_UPDATE_ERROR          0xF000001A  /** update packet format fail*/
#define   R1_E_GET_TIME_ERROR       		0xF0000010	/** get time fail*/
#define   R1_E_SHARING_VIOLATION       		0xF0000011	/** open device fail*/
#define   R1_E_ACCESS_DENIED         		0xF0000012	/** open device permission fail*/
#define   R1_E_CREATE_SHARE_M         		0xF0000013	/** create share memory fail*/
#define   R1_E_EXIST_SHARE_M          		0xF0000014	/** share memory exist*/
#define   R1_E_GET_RSA_KEY          		0xF0000015	/** get RSA key fail*/
#define   R1_E_RSA_NOT_PAIR          		0xF0000016	/** RSA key is not match*/
#define   R1_E_GEN_RSA_PEM          		0xF0000017	/** generate RSA PEM file fail*/
#define   R1_E_PEM_BITS_ERR          		0xF0000018	/** RSA PEM file does not match the number of bits set*/
#define   R1_E_RSA_KEY_1024          		0xF0000019	/** RSA key file must be 1024*/

//! The underlying device error

#define ERR_SUCCEF     		  0xd0//success
#define ERR_VERIFYUSERPIN	  0xd1//authentication user passwd fail
#define ERR_VERIFYSOPIN		  0xd2//authentication vendor passwd fail
#define ERR_NOUSRPERMISSION   0xd3//no usr permission
#define ERR_UPCOUNTUSRPIN     0xd4//more than usrpin a cap
#define ERR_NOSOPERMISSION    0xd5//no vendor permission
#define ERR_UPCOUNTSOPIN      0xd6//more than sopin a cap
#define ERR_UPREADSOMEMORY	  0xd7//read private memory out of bounds error
#define ERR_UPREADUSRMEMORY	  0xd8//read public memory out of bounds error
#define ERR_UPWRITESOMEMORY	  0xd9//write private memory out of bounds error
#define ERR_UPWRITEUSRMEMORY  0xda//write public memory out of bounds error
#define ERR_RESETSTATE  	  0xdb//reset anonymous state error
#define ERR_NODESKEY  	      0xdc//unknown DESKEYID
#define ERR_NOFILE			  0xdd//unknown file format
#define ERR_NOHID			  0xde//wrong HID
#define ERR_NOPID			  0xdf//wrong PID
#define ERR_UPDATE_TIME		  0xe0//update packet expired
#define ERR_PARAMETERS		  0xe1//parameter error
#define ERR_UNFINISHED		  0xe2//data packet recive incomplete
#define ERR_INACTIVE		  0xe3//COS no inactive
#define ERR_NOKEY			  0xe4//key no setup
#define ERR_AUTHORIZATION	  0xe5//Authorization has been exhausted
#define ERR_CLEANCOS          0xe6//clear COS error
#define ERR_PID_IS_F          0xe7//pid 8 byte are 0xff error
#define ERR_RSA_ENC_DATA      0xec//use 512 key data encryptdata per 64 byte's before 2 byte is not 0x00,use 1024 key data encryptdata per 128 byte's before 2 byte is not 0x00 error
#define ERR_CHECK_SUM		  0xed//check sum error

#define   R1_E_ERROR_TONGXIN				0xFFFFFFFE	/** communication error*/
#define   R1_E_ERROR_UNKNOWN				0xFFFFFFFF	/** unknown error*/

#define   MEM_PUB        0x00
#define   MEM_PRI        0x01

#define  LED_OFF         0x00
#define  LED_ON          0x01

#define  RSA_PAIRS_512   0x00
#define  RSA_PAIRS_1024  0x01

#define  PUB_KEY         0x00
#define  PRI_KEY         0x01


#define Update_TYPE_HID		 0x01	 //HID
#define Update_TYPE_PID		 0x02	 //PID
#define Update_TYPE_Count	 0x03	 //record times
#define Update_TYPE_PubKey   0x04   //public key data
#define Update_TYPE_PriKey   0x05   //private key data
#define Update_TYPE_3DesKey  0x06   //3DES key data
#define Update_TYPE_Memory   0x07   //8192 byte data section

#ifdef __cplusplus
extern "C" {
#endif

/**
* @brief  Find the encryption on the computer locks the specified PID
*         Back to find the specified number of PID's encryption locks
*
* @param  pid     [in]     Product Identification(8 characteristics,String range requirements is (0-9 a-f), pid = NULL, Back to the encryption of all locks on the computer
* @param  dwCount [out]    Back to find the number of encryption locks
*
* @return Reference the error code definition
*/
unsigned int R1_Find(BYTE* pid,unsigned int* dwCount);


/**
* @brief  Open the lock on the computer specified encryption
*
* @param  handle  [out]   Encryption lock handle
* @param  pid     [in]    encrypt lock PID(8 characteristics,String range requirements is (0-9 a-f), pid = NULL, Open the first to find the encryption locks
* @param  nIndex  [in]    Encrypted serial number lock
*
* @return Reference the error code definition
*/
unsigned int R1_Open(usb_dev_handle** handle,BYTE* pid,int nIndex);

/**
* @brief  Turn off the computer specify the encryption lock
*
* @param  handle  [in]    Encryption lock handle
*
* @return Reference the error code definition
*/
unsigned int R1_Close(usb_dev_handle* g_devicehandle);

/**
* @brief  PID specified in computer encryption locks on the version information
*
* @param  handle        [in]     Encryption lock handle
* @param  bVersionMajor [out]    Back to the main finding of the encrypted version lock
* @param  bVersionMinor [out]    Back to find a second version of the encrypted information lock
*
* @return Reference the error code definition
*/
unsigned int R1_GetVersion(usb_dev_handle* g_device_handle,BYTE* bVersionMajor,BYTE* bVersionMinor);

/**
* @brief  control LED.(The need for developers or user permission  )
*
* @param  handle     [in]     Encryption lock handle
* @param  bFlag      [in]     LED lights light off  LED_OFF is offLED_ON is light
*
* @return Reference the error code definition
*/
unsigned int R1_LEDControl(usb_dev_handle* g_device_handle,BYTE bFlag);


/**
* @brief  Regenerate encryption computer lock specified PID.The need for developers permission
*
* @param  handle  [in]     Encryption lock handle
* @param  seed    [in]     PID lock code encryption seed
* @param  len     [in]     The seeds of encryption code length lock PID   1~255
* @param  pid     [out]    encrypt lock PID(8 characters, requiring a string in the range 0-9 a-f)
*
* @return Reference the error code definition
*/
unsigned int R1_ProducePID(usb_dev_handle* g_device_handle,BYTE* seed,BYTE len,BYTE* pid);


/**
* @brief  Get a lock on the computer specified encryption HID.
*
* @param  handle  [in]     Encryption lock handle
* @param  hid     [out]    encrypt lock HID(16 characters, requiring a string in the range 0-9 a-f)
*
* @return Reference the error code definition
*/
unsigned int R1_GetHID(usb_dev_handle* g_device_handle,BYTE* hid);

/**
* @brief  Get a lock on the computer specified encryption PID.
*
* @param  handle  [in]     Encryption lock handle
* @param  pid     [out]    encrypt lock PID(8 characters, requiring a string in the range 0-9 a-f)
*
* @return Reference the error code definition
*/
unsigned int R1_GetPID(usb_dev_handle* g_device_handle,BYTE* pid);

/**
* @brief  Specify the encryption lock on the computer to verify the user password.
*
* @param  handle  [in]     Encryption lock handle
* @param  pin[in]     	   user passwd(Up to 255 characters, alphanumeric characters required)
* @param  tryCount[out]    Back to the user's remaining number of retries
*
* @return Reference the error code definition
*/
unsigned int R1_VerifyUserPin(usb_dev_handle* g_device_handle,BYTE* pin,BYTE* tryCount);

/**
* @brief  Modify the computer lock on the specified encrypted password. (anonymous permission)
*         When a developer permission passwords directly passed to the new user password
*
* @param  handle  [in]     Encryption lock handle
* @param  oldpin  [in]     old password(Up to 255 characters,alphanumeric characters required)
* @param  newpin  [in]     new password(Up to 255 characters,alphanumeric characters required)
*
* @return Reference the error code definition
*/
unsigned int R1_ChangeUserPin(usb_dev_handle* g_device_handle,BYTE* oldpin,BYTE* newpin);

/**
* @brief  Reset the computer locks the specified encrypted password. Developers need the password
*
* @param  handle  [in]     Encryption lock handle
*
* @return Reference the error code definition
*/
unsigned int R1_ResetUserPin(usb_dev_handle* g_device_handle);

/**
* @brief  Specify the encryption settings on the computer user's password lock retries.(The need for developers permission)
*         If you do not reset the number of calls to set the encryption lock default is 255, that there is no limit
*
* @param  handle  [in]     Encryption lock handle
* @param  TryCount[in]     Retries 0~2550 representative deadlock255 no limit
*
* @return Reference the error code definition
*/
unsigned int R1_SetTryCountForUserPin(usb_dev_handle* g_device_handle,BYTE TryCount);


/**
* @brief  Lock the computer to verify the developer specify the encryption password.
*
* @param  handle  [in]     Encryption lock handle
* @param  pin   [in]     developer password(16 characters, requiring a string in the range 0-9 a-f)
* @param  tryCount[out]    Returns the remaining number of retries developers
*
* @return Reference the error code definition
*/
unsigned int R1_VerifySoPin(usb_dev_handle* g_device_handle,BYTE* pin,BYTE* tryCount);

/**
* @brief  Password generated by seed developers(developer permission)
*
* @param  handle  [in]     Encryption lock handle
* @param  seed    [in]     Password generated seed developers
* @param  len     [in]     Seeds generated password length of developers 1~255
* @param  newpin  [out]    new developer password(16 characters, requiring a string in the range 0-9 a-f)
*
* @return Reference the error code definition
*/
unsigned int R1_ProduceSoPin(usb_dev_handle* g_device_handle,BYTE* seed,BYTE len,BYTE* newpin);


/**
* @brief  Generate random numbers (user permission)
*
* @param  handle  [in]     Encryption lock handle
* @param  buffer  [out]    Random number generated encryption locks(16 characters, requiring a string in the range 0-9 a-f)
*
* @return Reference the error code definition
*/
unsigned int R1_GenRandom(usb_dev_handle* g_device_handle,BYTE* buffer);

/**
* @brief  Set the computer locks the developer to specify encryption password retries (The need for developers permission)
*         If you do not reset the number of calls to set the encryption lock default is 255, that there is no limit
*
* @param  handle  [in]     Encryption lock handle
* @param  TryCount[in]     Retries 0~2550 representative deadlock255 no limit
*
* @return Reference the error code definition
*/
unsigned int R1_SetTryCountForSoPin(usb_dev_handle* g_device_handle,BYTE TryCount);

/**
* @brief  read lock on the computer specified in the specified memory encryption
*         Public memory and private memory are 0 ~ 4k
*         Developer can read and write permissions to the public memory, read and write private memory
*         Users can read and write permissions to the public memory, reading and private memory
*
* @param  handle  [in]     Encryption lock handle
* @param  wOffset [in]     start address
* @param  wLenght [in]     read length
* @param  buf     [out]    read data memory block
* @param  len     [out]    The actual size of the read memory
* @param  bType   [in]     Setting is to write public memory or private memory. read public memory bType = MEM_PUB.Otherwise,bType = MEM_PRI
*
* @return Reference the error code definition
*/
unsigned int R1_Read(usb_dev_handle* g_device_handle,WORD wOffset,WORD wLength,BYTE* buf,WORD* len,BYTE bType);


/**
* @brief  Write lock on the computer specified in the specified memory encryption
*         Public memory and private memory are 0 ~ 4k
*         Developer can read and write permissions to the public memory, read and write private memory
*         Users can read and write permissions to the public memory, reading and private memory
*
* @param  handle  [in]     Encryption lock handle
* @param  wOffset [in]     start address
* @param  wLenght [in]     write length
* @param  buf     [out]    write data memory block
* @param  len     [out]    The actual size of the write memory
* @param  bType   [in]     Setting is to write public memory or private memory. write public memory bType = MEM_PUB.Otherwise,bType = MEM_PRI
*
* @return Reference the error code definition
*/
unsigned int R1_Write(usb_dev_handle* g_device_handle,WORD wOffset,WORD wLength,BYTE* buf,WORD* len,BYTE bType);


/**
* @brief  :Reset the lock's security status, return to anonymity.
*
* @param  handle  [in]     Encryption lock handle
*
* @return Reference the error code definition
*/
unsigned int R1_ResetSecurityState(usb_dev_handle* g_device_handle);


/**
* @brief  setup 3DES key (The need for developers permission)
*
* @param  handle  [in]     Encryption lock handle
* @param  id      [in]     3DES key ID   0~7
* @param  buf     [in]     key (Fixed length of 16 bytes)
*
* @return Reference the error code definition
*/
unsigned int R1_SetTDesKey(usb_dev_handle* g_device_handle,BYTE id,BYTE* buf);

/**
* @brief  using 3DES key encrypt (Requires user privileges)
*
* @param  handle  [in]     Encryption lock handle
* @param  id      [in]     3DES key ID   0~7
* @param  buf     [in,out] encrypt contents
* @param  dwLen   [in]     encrypt contents length (Must be divisible by 8)
*
* @return Reference the error code definition
*/
unsigned int R1_TDesEnc(usb_dev_handle* g_device_handle,BYTE id,BYTE* buf,unsigned int dwLen);

/**
* @brief  using 3DES key decrypt (Requires user privileges)
*
* @param  handle  [in]     Encryption lock handle
* @param  id      [in]     3DES key ID   0~7
* @param  buf     [in]     decrypt contents
* @param  dwLen   [in]     decrypt contents length (Must be divisible by 8)
*
* @return Reference the error code definition
*/
unsigned int R1_TDesDec(usb_dev_handle* g_device_handle,BYTE id,BYTE* buf,unsigned int dwLen);

/**
* @brief  Generate RSA key pair (The need for developers permission)
*
* @param  handle  [in]     Encryption lock handle
* @param  bFlag   [in]     The length of the key pair generated  RSA_PAIRS_512 representative 512RSA_PAIRS_1024 representative 1024  
* @param  id      [in]     Id generated key pair    0~8 Ninth upgrade RSA key pair that
* @param  pubkey  [out]    Generated public key file path
* @param  prikey  [out]    Generated private key file path
*
* @return Reference the error code definition
*/
unsigned int R1_GenRSAKey(usb_dev_handle* g_device_handle,BYTE bFlag,BYTE id,char* pubkey,char* prikey);

/**
* @brief  setup RSA key pair(The need for developers permission)
*
* @param  handle  [in]     Encryption lock handle
* @param  bFlag   [in]     setup key pair length RSA_PAIRS_512 representative 512 bitRSA_PAIRS_1024 representative 1024 bit
* @param  id      [in]     RSA key ID   0~8 Ninth upgrade RSA key pair that,Upgrading RSA 1024-bit key pair must be
* @param  pubkey  [in]     Public key file path set
* @param  prikey  [in]     Set the private key file path
*
* @return Reference the error code definition
*/
unsigned int R1_SetRSAKey(usb_dev_handle* g_device_handle,BYTE bFlag,BYTE id,char* pubkey,char* prikey);

/**
* @brief   Decrypt using RSA key pair (Requires user privileges)
* Limited by the RSA algorithm itself. If you are 512 key pairs, each 64 bytes of the first two bytes must be 0 if the key pair is 1024, each 128 bytes of the first two bytes must be 0
*
* @param  handle  [in]     Encryption lock handle
* @param  bFlag   [in]     using public key or private key encrypt PUB_KEY representative using public keyPRI_KEY representative using private key
* @param  id      [in]     RSA key ID   0~7
* @param  bBuf    [in,out] encrypt contents
* @param  dwLen   [in]     encrypt contents length length must be 128 times
*
* @return Reference the error code definition
*/
unsigned int R1_RSAEnc(usb_dev_handle* g_device_handle,BYTE bFlag,BYTE id,BYTE* bBuf,unsigned int dwLen);

/**
* @brief  Decrypt using RSA key pair (Requires user privileges)
* Limited by the RSA algorithm itself. If you are 512 key pairs, each 64 bytes of the first two bytes must be 0 if the key pair is 1024, each 128 bytes of the first two bytes must be 0
*
* @param  handle  [in]     Encryption lock handle
* @param  bFlag   [in]     using public key or private key decrypt PUB_KEY representative using public keyPRI_KEY representative using private key
* @param  id      [in]     RSA key ID   0~7
* @param  bBuf    [in,out] decrypt contents
* @param  dwLen   [in]     decrypt contents length length must be 128 times Whether it is a 1024 bit key or 512 key
*
* @return Reference the error code definition
*/
unsigned int R1_RSADec(usb_dev_handle* g_device_handle,BYTE bFlag,BYTE id,BYTE* bBuf,unsigned int dwLen);



/**
* @brief  Get total number of sub-modules (The need for developers permission)
*
* @param  handle  [in]     Encryption lock handle
* @param  bCount  [in]     modules count 0~255 0 representative deadlock255 representative no limit
*
* @return Reference the error code definition
*/
unsigned int R1_SetCounter(usb_dev_handle* g_device_handle,BYTE bCount);

/**
* @brief  Get total number of sub-modules (Requires user privileges)
*
* @param  handle  [in]     Encryption lock handle
* @param  bCount  [in]     modules count 0~255 0 representative deadlock255 representative no limit
*
* @return Reference the error code definition
*/
unsigned int R1_GetCounter(usb_dev_handle* g_device_handle,BYTE* bCount);


/**
* @brief  Set the update data (anonymous permission)
*
* @param  UpdateType		[in]   Types need to be upgraded
* @param  wFirst        	[in]   When the representative of binding HIDPID:No effect.when upgrading the record time:Representative of record times.When upgrading memory:Representative offset.When the upgrade RSA key pair:Representative RSA type.0 representative 5121 representative 1024.When the upgrade 3DES key pair:no effectsend 0
* @param  wSecond       	[in]   When the representative of binding HIDPID:No effect.when upgrading the record time:No effectsend 0.When upgrading memory:Upgrading the length of the representative.When the upgrade RSA key pair:Representative key pair ID.When the upgrade 3DES key pair:representative key ID
* @param  pBuf 				[in]   Update the data produced
* @param  Len				[in]   Making the length of the data update
*
* @return Reference the error code definition
*/
unsigned int R1_SetUpdatePacket(unsigned int UpdateType,WORD wFirst,WORD wSecond,BYTE* pBuf, unsigned int Len);

/**
* @brief  Production update (anonymous permission)
*
* @param  pubkey  [in]      To create the key file updates
* @param  pEncBuf [out]     Ciphertext making updates to apply for the length of not less than 8k of memory
* @param  pEncLen [out]     Production updates the length of the ciphertext
*
* @return Reference the error code definition
*/
unsigned int R1_GenUpdatePacket(char* pubkey,BYTE* pEncBuf,WORD* pEncLen);

/**
* @brief  update (anonymous permission)
*
* @param  handle  [in]     Encryption lock handle
* @param  bPacket [in]     Ciphertext update
* @param  wLen    [in]     Update the length of the ciphertext
*
* @return Reference the error code definition
*/
unsigned int R1_Update(usb_dev_handle* g_device_handle,BYTE* bPacket,unsigned short wLen);

/**
* @brief  Get an error message based on error codes (anonymous permission)
* @param  dwError [in]     error code
* @return Reference the error code definition
*/
char* R1_GetErrInfo(unsigned int dwError);

#ifdef __cplusplus
}
#endif

#endif //! ROCKEY1_API


