#ifndef ROCKEY1_COMMAND_H
#define ROCKEY1_COMMAND_H
#include <stdio.h>
#include "datatype.h"
#include "libusb/usb.h"
#include "libusb/usbi.h"


//! The underlying device error

#define ERR_SUCCEF     		  0xd0//success
#define ERR_VERIFYUSERPIN	  0xd1//authentication user passwd fail
#define ERR_VERIFYSOPIN		  0xd2//authentication vendor passwd fail
#define ERR_NOUSRPERMISSION   0xd3//no usr permission
#define ERR_UPCOUNTUSRPIN     0xd4//more than usrpin a cap
#define ERR_NOSOPERMISSION    0xd5//no vendor permission
#define ERR_UPCOUNTSOPIN      0xd6//more than sopin a cap
#define ERR_UPREADSOMEMORY	  0xd7//read private memory out of bounds error
#define ERR_UPREADUSRMEMORY	  0xd8//read public memory out of bounds error
#define ERR_UPWRITESOMEMORY	  0xd9//write private memory out of bounds error
#define ERR_UPWRITEUSRMEMORY  0xda//write public memory out of bounds error
#define ERR_RESETSTATE  	  0xdb//reset anonymous state error
#define ERR_NODESKEY  	      0xdc//unknown DESKEYID
#define ERR_NOFILE			  0xdd//unknown file format
#define ERR_NOHID			  0xde//wrong HID
#define ERR_NOPID			  0xdf//wrong PID
#define ERR_UPDATE_TIME		  0xe0//update packet expired
#define ERR_PARAMETERS		  0xe1//parameter error
#define ERR_UNFINISHED		  0xe2//data packet recive incomplete
#define ERR_INACTIVE		  0xe3//COS no inactive
#define ERR_NOKEY			  0xe4//key no setup
#define ERR_AUTHORIZATION	  0xe5//Authorization has been exhausted
#define ERR_CLEANCOS          0xe6//clear COS error
#define ERR_PID_IS_F          0xe7//pid 8 byte are 0xff error
#define ERR_RSA_ENC_DATA      0xec//use 512 key data encryptdata per 64 byte's before 2 byte is not 0x00,use 1024 key data encryptdata per 128 byte's before 2 byte is not 0x00 error
#define ERR_CHECK_SUM		  0xed//check sum error

//! The underlying device command set
#define   R1ClearCos                          0x00/**  clear device COS*/
#define   R1LEDControl                        0x01/** control device LED*/
#define   R1Read                              0x02/** read device*/
#define   R1Write                             0x03/** write device*/
#define   R1GenRandom                         0x04/** generate random*/
#define   R1SeedTransform                     0x05/** Transform*/
#define   R1GetHID                            0x06/** get device HID*/
#define   R1GetPID                            0x07/** get device PID*/
#define   R1GenPID                            0x08/** generate PID*/
#define   R1VerifyUserPin                     0x09/** confirm USER PIN*/
#define   R1ChangeUserPin                     0x0A/** change USER PIN*/
#define   R1SetTryCountForUserPin             0x0B/** Reset retries*/
#define   R1VerifySoPin                       0x0C/** confirm SO PIN*/
#define   R1GenSoPin                          0x0D/** generate SO PIN*/
#define   R1SetTryCountForSoPin               0x0E/** Reset retries*/
#define   R1ResetSecurityState                0x0F/** reset anonymous state*/
#define   R1GenRSAKey                         0x10/** generate RSA key pair*/
#define   R1SetRSAKey                         0x11/** setup RSA key pair*/
#define   R1RSAPubEncrypt                     0x12/** public key encrypt*/
#define   R1RSAPriDecrypt                     0x13/** private key decrypt*/
#define   R1SetTDESKey                        0x14/** setup 3DES KEY*/
#define   R1TDES                              0x15/** 3DES operation*/
#define   R1SetCounter                        0x16/** setup counter*/
#define   R1GenUpdatePacket                   0x17/** generate update packet*/
#define   R1Update                            0x18/** update*/
#define   R1SessionKeysExchange               0x19/** exchange key pair*/
#define   R1SetHID                            0x1A/** setup device HID*/
#define   R1ActationCos                       0x1B/** active device COS*/
#define   R1SetSession_RSAKey                 0x1C /** Set encryption key used by the RSA key pair*/
#define   R1GetCounter                        0x1D /** Access to the remaining number of encryption locks*/
#define   R1GetVersion                        0x1E /** Obtain the encrypted version lock*/
#define   R1ResetUserPin                      0x1F /** Reset user password encryption locks*/
#define   RIS_GETRSA_KEY                      0x23 //get RSA key
#define   RIS_GETRANDOM_KEY                   0x24 //get exchange key
#define   RIS_FLASHRSA_KEY                    0x25 //Stored in FLASH access key 9

#define  PIN_LENGTH      0x08

typedef struct __attribute__((aligned(1)))
{
	BYTE    id;
	BYTE	command[5];
	BYTE	data[19];
} Feature_Report;


typedef struct __attribute__((aligned(1)))
{
	BYTE  bFlag[4];  //! These four bytes are the symbol of shared memory
	BYTE  bExist;    //! 0 indicates that there is no shared memory, shared memory, said there was a
	BYTE  bKey[8];   //! random key

} ShareM;


typedef struct __attribute__((aligned(1)))
{
	BYTE bSign[4];                                /** update file logo.0x4654524b,*/
	BYTE bVersion[2];                             /** Update file version information. The first byte is the major version number, the second byte is the minor version number*/
	BYTE bBind;                                   /** Upgrade file binding information. 0x00 nothing binding. Binding binding HID.0x02 PID 0x01*/
	BYTE bReserve;                                /** reserved*/
	BYTE bHid[8];                                 /** HID upgrade file to bind*/
	BYTE bPid[8];                                 /** Upgrade file to bind PID*/
	unsigned int  dwDate;                         /** date*/
} UpdateFileHeader;

typedef struct __attribute__((aligned(1)))
{
	BYTE bFlag;                                  /** 0 representatives 5121 representatives 1024*/
	BYTE bID;                                     /** If the RSA key pair, the highest representative of a 512-bit = 0, = 1 for 1024. The remaining seven representatives of RSA key pair number (0 to 7)*/
	BYTE bPubkey[256];                           /** Public key file name*/
	BYTE bPrikey[256];                           /** Private key file name*/
} RSAKey;

typedef struct __attribute__((aligned(1)))
{
	BYTE bID;                                     /** If the 3DES key pair on behalf of 3DES key serial number0~7*/
	BYTE bTDESkey[16];
}TDESKey;

typedef struct __attribute__((aligned(1)))
{
	BYTE bType;                                   /** Upgrading the type of document. Said private memory upgrade 0x01, 0x02, said total time to upgrade, 0x04, said upgraded RSA key pair, 0x08 3DES key for that upgrade*/
	WORD wMemOffset;                              /** To upgrade the memory offset address private*/
	WORD wMemLength;                              /** To upgrade the length of the private memory*/
	BYTE bMem[4*1024];                            /** The contents of private memory*/
	BYTE bCounter;                                /** Total time unit. 0 for deadlock, and 255 for no restriction*/
	BYTE bRSAKeyCount;                            /** RSA Number of key pair*/
	RSAKey pRSAkey[8];                            /** RSA key pair*/
	BYTE bTDESKeyCount;                           /** 3DES Number of key pair*/
	TDESKey pTDESKey[8];                          /** 3DES key*/
} UpdateFileContent;

typedef struct __attribute__((aligned(1)))
{
	UpdateFileHeader ufh;                         /** updata packet head*/
	UpdateFileContent ufc;                        /** updata packet contents*/
} UpdatePacket;

#ifdef __cplusplus
extern "C"
{
#endif

	/**
	* @brief  Find encryption lock, this function directly to return handle encryption and the number of locks, the handle is no need, to deal with their own closed. (anonymous can)
	*
	* @param  handle  [in]     encrypt lock handle
	* @param  dwCount [in]     encrypt lock counter
	*
	* @return R1_S_SUCCESS: execute success; FT_PARAMETER_ERROR:parameter error
	*/
	unsigned int R1_FindEx(usb_dev_handle** g_device_handle,unsigned int* dwCount);

	/**
	* @brief  clear COS(anonymous state)
	*
	* @param  handle  [in]     encrypt lock handle
	*
	* @return R1_S_SUCCESS: execute success; FT_PARAMETER_ERROR:parameter error
	*/
	unsigned int R1_ClearCos(usb_dev_handle* g_device_handle);

	/**
	* @brief  active COS(anonymous state)
	*
	* @param  handle  [in]     encrypt lock handle
	*
	* @return R1_S_SUCCESS: execute success; FT_PARAMETER_ERROR:parameter error
	*/
	unsigned int R1_ActationCoss(usb_dev_handle* g_device_handle);

	/**
	* @brief  setup communicate key(anonymous state)
	*
	* @param  handle  [in]     encrypt lock handle
	* @param  pubkey  [in]     Public key file path
	* @param  prikey  [in]     Private key file path
	*
	* @return R1_S_SUCCESS: execute success; FT_PARAMETER_ERROR:parameter error
	*/
	unsigned int R1_SetCommunicateKey(usb_dev_handle* g_device_handle,char* pubkey,char* prikey);

	/**
	* @brief  exchange communicate key(anonymous state)
	*
	* @param  handle  [in]     encrypt lock handle
	* @param  pubkey  [out]    send communicate key
	*
	* @return R1_S_SUCCESS: execute success; FT_PARAMETER_ERROR:parameter error
	*/
	unsigned int R1_ExchangeCommunicateKey(usb_dev_handle* g_device_handle,BYTE* key);

	/**
	* @brief  Specify the encryption settings on the computer locks the HID.
	*
	* @param  handle  [in]     encrypt lock handle
	* @param  hid     [out]    encrypt lock HID(16 characteristics)
	*
	* @return R1_S_SUCCESS: execute success; FT_PARAMETER_ERROR:parameter error
	*/
	unsigned int R1_SetHID(usb_dev_handle* g_device_handle,BYTE* hid);

	/**
	* @brief  Generate a new seed by seed code (User rights or permission to developers)
	*
	* @param  handle  [in]     encrypt lock handle
	* @param  seed    [in]     Generate a new code of seed seed
	* @param  len     [in]     Students generate a new seed of the length of the seed code 1~255
	* @param  newbuf  [out]    New seed code(16 byte)
	*
	* @return R1_S_SUCCESS: execute success; FT_PARAMETER_ERROR:parameter error
	*/
	unsigned int R1_Transform(usb_dev_handle* g_device_handle,BYTE* seed,BYTE len,BYTE* newbuf);


	/**
	* @brief  Set the computer to get a lock on the specified RSA key encryption.
	*
	* @param  handle  [in]     encrypt lock handle
	* @param  key     [out]    encrypt lock rsakey(256 byte)
	*
	* @return R1_S_SUCCESS: execute success; FT_PARAMETER_ERROR:parameter error
	*/
	unsigned int R1_GetRSAKey(usb_dev_handle* g_device_handle,BYTE* key);

	/**
	* @brief  Set the computer to get a lock on the specified RSA key encryption.
	*
	* @param  handle  [in]     encrypt lock handle
	* @param  key     [out]    encrypt lock radomkey(8 byte)
	*
	* @return R1_S_SUCCESS: execute success; FT_PARAMETER_ERROR:parameter error
	*/
	unsigned int R1_GetRandomKey(usb_dev_handle* g_device_handle,BYTE* key);

	/**
	* @brief  Set the computer to get a lock on the specified flash rsa encryption key.
	*
	* @param  handle  [in]     encrypt lock handle
	* @param  key     [out]    encrypt lock flashrsakey(8 byte)
	*
	* @return R1_S_SUCCESS: execute success; FT_PARAMETER_ERROR:parameter error
	*/
	unsigned int R1_GetFlashRSAKey(usb_dev_handle* g_device_handle,BYTE* key);
#ifdef __cplusplus
}
#endif

#endif //!ROCKEY1_COMMAND_H


