#ifndef _DRV_H
#define _DRV_H

#include "Rockey1_API.h"
#include "datatype.h"
#include "openssl/sha.h"

#include "Rockey1_command.h"


#ifdef __cplusplus
extern "C"
{
#endif
#define HID_REPORT_GET       0x01
#define HID_REPORT_SET       0x09

#define R1_MAX         128 
#define R1_VID         0x096e
#define R1_PID         0x0205
#define CMDREPORTLEN   25

//share memory macro
#define SHM_SIZE       1024
#define SHM_MODE       0666

#define EPSERR_SUCCESS							0
#define EPSERR_NO_SUCH_DEVICE						0xA0100001
#define EPSERR_NOT_OPENED_DEVICE					0xA0100002
#define EPSERR_WRONG_UID						0xA0100003
#define EPSERR_INDEX							0xA0100004
#define EPSERR_TOO_LONG_SEED						0xA0100005
#define EPSERR_WRITE_PROTECT						0xA0100006
#define EPSERR_OPEN_DEVICE						0xA0100007
#define EPSERR_READ_REPORT						0xA0100008
#define EPSERR_WRITE_REPORT						0xA0100009
#define EPSERR_SETUP_DI_GET_DEVICE_INTERFACE_DETAIL			0xA010000A
#define EPSERR_GET_ATTRIBUTES						0xA010000B
#define EPSERR_GET_PREPARSED_DATA					0xA010000C
#define EPSERR_GETCAPS							0xA010000D
#define EPSERR_FREE_PREPARSED_DATA					0xA010000E
#define EPSERR_FLUSH_QUEUE						0xA010000F
#define EPSERR_SETUP_DI_CLASS_DEVS					0xA0100010
#define EPSERR_GET_SERIAL						0xA0100011
#define EPSERR_GET_PRODUCT_STRING					0xA0100012
#define EPSERR_TOO_LONG_DEVICE_DETAIL					0xA0100013
#define EPSERR_UNKNOWN_DEVICE						0xA0100020
#define EPSERR_VERIFY							0xA0100021
#define EPSERR_ERASE							0xA0100022
#define EPSERR_UNKNOWN_ERROR						0xA010FFFF

#pragma pack(1)
	typedef struct
	{
		struct usb_device* udev;
		usb_dev_handle *device_handle;
		char   device_path[256];
		BYTE	pid[20];
		BYTE    hid[20];
		BYTE*   pShareM;
	} G_DevInfo;
#pragma pack()
	

	int FindR1(unsigned int * nCount_device);

	int FindR1_EX(usb_dev_handle** g_device_handle,unsigned int* dwCount);

	/**
	* @brief  Find the computer locks the specified encrypted PID.
	*
	* @param  handle  [in]     encrypt lock handle
	* @param  pid     [out]    encrypt lock PID(8 characteristics)
	*
	* @return R1_S_SUCCESS: execute success; FT_PARAMETER_ERROR:parameter error
	*/
	unsigned int R1_GetPID_EX(usb_dev_handle* g_device_handle,BYTE* pid);

	/**
	* @brief  
	*
	* @param  hHidDev      [in] 	   encrypt lock handle
	* @param  pSendBuffer  [in]       Issued the command string address
	* @param  pRcvBuffer   [out] 	  Accept the command string address
	*
	* @return 0: execute success; -1 encrypt lock not exist; -2 check error
	*/
	int CommandReport(usb_dev_handle* g_device_handle, void* pSendBuffer, void* pRcvBuffer);

	//! This library provides three functions for the service of shared memory
	BYTE* CreateShareM(char* device_path,BYTE* hid,BYTE* bExist);//! bExist = 0 Representative re-create=1 Representatives already exists
	void  CloseShareM(BYTE* pBuf);
	BOOL  IfExistCommuniteKey(BYTE* pShare);
	void  ClearCommuniteKey(usb_dev_handle* g_device_handle);
	void  SetCommuniteKey(BYTE* pShare);
	void  GetCommuniteKey(usb_dev_handle* g_device_handle,BYTE* pShare);

#ifdef __cplusplus
}
#endif




#endif //!_DRV_H
